package app.alan.alan_voice

import io.flutter.plugin.common.MethodCall
import io.flutter.plugin.common.MethodChannel
import io.flutter.plugin.common.MethodChannel.MethodCallHandler
import io.flutter.plugin.common.MethodChannel.Result
import io.flutter.plugin.common.PluginRegistry.Registrar
import android.view.View
import android.view.Gravity
import android.widget.FrameLayout
import android.app.Activity
import android.content.res.Resources
import android.view.ViewGroup
import com.alan.alansdk.Alan
import com.alan.alansdk.AlanCallback
import com.alan.alansdk.BasicSdkListener
import com.alan.alansdk.ScriptMethodCallback
import com.alan.alansdk.alanbase.DialogState
import com.alan.alansdk.button.AlanButton
import com.alan.alansdk.events.EventCommand
import io.flutter.plugin.common.EventChannel

class AlanVoicePlugin(registrar: Registrar): MethodCallHandler {

  private val activity: Activity = registrar.activity()
  private var alanButton: AlanButton? = null
  private var callBackChannel: EventChannel = EventChannel(registrar.messenger(), "alan_voice_callback")

  init {


  }
  companion object {

    private val ARGUMENT_PROJECT_SERVER = "projectServer"
    private val ARGUMENT_PROJECT_ID = "projectId"
    private val ARGUMENT_PROJECT_DIALOG_ID = "projectDialogId"
    private val ARGUMENT_PROJECT_AUTH_JSON = "projectAuthJson"

    private val ARGUMENT_METHOD_NAME = "method_name"
    private val ARGUMENT_METHOD_ARGS = "method_args"

    private val ARGUMENT_VISUALS = "visuals"

    @JvmStatic
    fun registerWith(registrar: Registrar) {
      val channel = MethodChannel(registrar.messenger(), "alan_voice")
      channel.setMethodCallHandler(AlanVoicePlugin(registrar))
    }
  }

  override fun onMethodCall(call: MethodCall, result: Result) {
    when (call.method) {
      "getVersion" -> version(result)
      "createButton" -> createButton(result)
      "initButton" -> initButton(result, call)
      "showButton" -> showButton(result)
      "callScript" -> callScript(result, call)
      "setVisuals" -> setVisuals(result, call)
      else -> result.notImplemented()
    }
  }

  private fun setVisuals(result: Result, call: MethodCall) {
    if (alanButton != null) {
      alanButton?.sdk?.setVisuals(call.argument<String>(ARGUMENT_VISUALS))
      result.success(true)
    } else {
      result.error("No alan button", null, null)
    }
  }

  private fun callScript(result: Result, call: MethodCall) {
    if (alanButton != null) {
      alanButton?.sdk?.call(call.argument<String>(ARGUMENT_METHOD_NAME),
              call.argument<String>(ARGUMENT_METHOD_ARGS)
      ) { method, body, error ->
          result.success(listOf(method, body, error))
      }
    } else {
      result.error("No alan button", null, null)
    }
  }

  private fun version(result: Result) {
    if (alanButton != null) {
      result.success(alanButton?.sdk?.version ?: "")
    } else {
      result.error("No alan button", null, null)
    }
  }

  private fun showButton(result: Result) {
    alanButton?.showButton()
    result.success(true)
  }

  private fun initButton(result: Result, call: MethodCall) {
    if (alanButton == null) {
      createButton(null)
    }
    val projectId = call.argument<String>(ARGUMENT_PROJECT_ID)
    val dialogId = call.argument<String>(ARGUMENT_PROJECT_DIALOG_ID)
    val server = call.argument<String>(ARGUMENT_PROJECT_SERVER)
    val authJson = call.argument<String>(ARGUMENT_PROJECT_AUTH_JSON)

    if (projectId != null) {
      alanButton?.initSDK(server, projectId, dialogId, authJson)
      subscribe(alanButton?.sdk)
      alanButton?.showButton()
      result.success(true)
    } else {
      result.error("No objectId, please provide objectId argument", null, null)
    }
  }

  inner class AlanEventSink : EventChannel.StreamHandler {
    private var sink : EventChannel.EventSink? = null

    override fun onListen(p0: Any?, p1: EventChannel.EventSink?) {
      sink = p1
    }

    override fun onCancel(p0: Any?) {
      sink = null
    }

    fun newDialogState(state: DialogState) {
      sink?.success(listOf("dialog_state_changed", state.name))
    }

    fun newCommand(payload: String) {
      sink?.success(listOf("command", payload))
    }
  }

  private fun subscribe(sdk: Alan?) {
    sdk?: return
    val alanSink = AlanEventSink()
    callBackChannel.setStreamHandler(alanSink)

    sdk.registerCallback(object : BasicSdkListener() {
      override fun onDialogStateChanged(dialogState: DialogState) {
        alanSink.newDialogState(dialogState)
      }

      override fun onCommandReceived(eventCommand: EventCommand?) {
        alanSink.newCommand(eventCommand?.data?.toString() ?: "")
      }
    })
  }

  private fun createButton(result: Result?) {
    val rootView = activity.findViewById(android.R.id.content) as ViewGroup

    alanButton = AlanButton(activity, null)
    val params = FrameLayout.LayoutParams(
            FrameLayout.LayoutParams.MATCH_PARENT, FrameLayout.LayoutParams.WRAP_CONTENT)
    params.gravity = Gravity.BOTTOM or Gravity.END
    alanButton?.let {
      it.layoutParams = params
      it.visibility = View.GONE
      it.isClickable = true
      rootView.addView(it)
    }

    result?.success(true)
  }
}
